#include "mesh.h"
#include <stdio.h>

#include <map>

void  CMesh::loadObjFile(std::string& filename)
{
  FILE  *objFile = fopen(filename.c_str(), "r");
  char line[160];
  char  tag[32], parameter[128];

  if (!objFile)
    return;

  while (fgets(line, 160, objFile))
  {
    // test for single comment lines
    if(line[0] == '#') continue;

    // set all characters in the parser strings to 0
    memset(parameter, 0, 128);
    memset(tag, 0, 32);
    sscanf(line, "%s %128c", tag, parameter);

    // Vertex
    if (std::string(tag) == "v")
    {
      CMesh::Vertex v;
      sscanf(parameter, "%lf %lf %lf", &(v.x), &(v.y), &(v.z));
      m_vectorVertices.push_back(v);
    }

    // Face
    else if(std::string(tag) == "f")
    {
      CMesh::Face f;
      if(std::string(parameter).find('/',std::string(parameter).find('/')+1) == std::string(parameter).find('/') + 1)
      {
        sscanf(parameter, "%d//%d %d//%d %d//%d", &f.v1, &f.n1, &f.v2, &f.n2, &f.v3, &f.n3);
        f.t1 = f.t2 = f.t3 = 0;
      }
      else if(std::string(parameter).find('/') != std::string::npos
        && std::string(parameter).find('/',std::string(parameter).find('/')+1) <
        std::string(parameter).find(' '))
      {
        sscanf(parameter, "%d/%d/%d %d/%d/%d %d/%d/%d", &f.v1, &f.t1, &f.n1, &f.v2, &f.t2, &f.n2, &f.v3, &f.t3, &f.n3);
      }
      else if(std::string(parameter).find('/') != std::string::npos
        && std::string(parameter).find('/',std::string(parameter).find('/')+1) >
        std::string(parameter).find(' '))
      {
        sscanf(parameter, "%d/%d %d/%d %d/%d", &f.v1, &f.t1, &f.v2, &f.t2, &f.v3, &f.t3);
        f.n1 = f.n2 = f.n3 = 0;
      }
      else
      {
        sscanf(parameter, "%d %d %d", &f.v1, &f.v2, &f.v3);
        f.n1 = f.n2 = f.n3 = 0;
        f.t1 = f.t2 = f.t3 = 0;
      }
      //Caution: Indices in .obj range from 1 to n while the vector class uses indices 0..n-1. Thus decrement all indices in the face.
      f.v1--; f.v2--; f.v3--; 
      f.t1--; f.t2--; f.t3--;
      f.n1--; f.n2--; f.n3--;
      m_vectorFaces.push_back(f);
    }
    else if (std::string(tag) == "vn")
    {
      CMesh::Vertex n;
      sscanf(parameter, "%lf %lf %lf", &(n.x), &(n.y), &(n.z));
      m_vectorNormals.push_back(n);
    }
    else if (std::string(tag) == "vt")
    {
      CMesh::TextureCoordinate t;
      sscanf(parameter, "%f %f", &(t.u), &(t.v));
      m_vectorTextureCoordinates.push_back(t);
    }
    else
    {
      // treat other tags
	 }
  }

  // calculate face normal
  for(size_t faceId = 0; faceId < m_vectorFaces.size(); ++faceId)
  {
	  CMesh::Face& face = m_vectorFaces[faceId];
	  CMesh::Vertex& v1 = m_vectorVertices[face.v1];
	  CMesh::Vertex& v2 = m_vectorVertices[face.v2];
	  CMesh::Vertex& v3 = m_vectorVertices[face.v3];

	  face.normal = cross( v2 - v1, v3 -v1).normalize();
  }

  // calculate vertex normals, if not present
  if(m_vectorNormals.empty())
  {
	  m_vectorNormals.resize(m_vectorVertices.size(), gmVector3(0,0,0));
	  for(size_t i = 0; i < m_vectorFaces.size(); ++i)
	  {
		  CMesh::Face& face = m_vectorFaces[i];
		  m_vectorNormals[face.v1] += face.normal;
		  m_vectorNormals[face.v2] += face.normal;
		  m_vectorNormals[face.v3] += face.normal;

		  face.n1 = face.v1;
		  face.n2 = face.v2;
		  face.n3 = face.v3;
	  }
	  for(size_t i = 0; i < m_vectorNormals.size(); ++i)
		  m_vectorNormals[i].normalize();
  }

  std::map<Edge, int> mapEdges;
  std::pair<std::map<Edge, int>::iterator,bool> res;

  for(size_t i = 0; i < m_vectorFaces.size(); ++i)
  {
	  CMesh::Face& face = m_vectorFaces[i];
	  {
		  for(int j = 0; j < 3; ++j)
		  {
			  Edge e;
			  switch(j)
			  {
			  case 0:
				  e.v0Id = face.v1; e.v1Id = face.v2;
				  break;
			  case 1:
				  e.v0Id = face.v2; e.v1Id = face.v3;
				  break;
			  case 2:
				  e.v0Id = face.v3; e.v1Id = face.v1;
				  break;
			  }

			  res = mapEdges.insert(std::pair<Edge, int>(e, -1));
			  if(res.second)
			  {
				  res.first->second = m_vectorEdges.size();
				  m_vectorEdges.push_back(e);
			  }

			  // get the edge from the edge vector and add the face neighbor
			  Edge& edge = m_vectorEdges[res.first->second];
			  if(edge.neightborFace[0] == -1)
				  edge.neightborFace[0] = i;
			  else
				  edge.neightborFace[1] = i;
		  }
	  }
  }

  fclose(objFile);
}
